ace.define("ace/snippets/makefile.snippets",["require","exports","module"],function(e,i,t){t.exports="snippet ifeq\n\tifeq (${1:cond0},${2:cond1})\n\t\t${3:code}\n\tendif\n"}),ace.define("ace/snippets/makefile",["require","exports","module","ace/snippets/makefile.snippets"],function(e,i,t){"use strict";i.snippetText=e("./makefile.snippets"),i.scope="makefile"}),ace.require(["ace/snippets/makefile"],function(e){"object"==typeof module&&"object"==typeof exports&&module&&(module.exports=e)});