ace.define("ace/mode/rst_highlight_rules",["require","exports","module","ace/lib/oop","ace/lib/lang","ace/mode/text_highlight_rules"],function(t,l,g){"use strict";var r=t("../lib/oop"),x=(t("../lib/lang"),t("./text_highlight_rules").TextHighlightRules),n=function(){var e_list="markup.heading",e_link="markup.underline.list",e_literal="support.function",i="(^|\\s|[\"'(<\\[{\\-/:])",o="(?:$|(?=\\s|[\\\\.,;!?\\-/:\"')>\\]}]))";this.$rules={start:[{token:"markup.heading",regex:"(^)([\\=\\-`:\\.'\"~\\^_\\*\\+#])(\\2{2,}\\s*$)"},{token:["text","keyword.operator",e_literal],regex:"(^\\s*\\.\\. )([^: ]+::)(.*$)",next:"codeblock"},{token:"keyword.operator",regex:"::$",next:"codeblock"},{token:["string",e_link],regex:"(^\\.\\. _[^:]+:)(.*$)"},{token:["string",e_link],regex:"(^__ )(https?://.*$)"},{token:"string",regex:"^\\.\\. \\[[^\\]]+\\] "},{token:"comment",regex:"^\\.\\. .*$",next:"comment"},{token:e_list,regex:"^\\s*[\\*\\+-] "},{token:e_list,regex:"^\\s*(?:[A-Za-z]|[0-9]+|[ivxlcdmIVXLCDM]+)\\. "},{token:e_list,regex:"^\\s*\\(?(?:[A-Za-z]|[0-9]+|[ivxlcdmIVXLCDM]+)\\) "},{token:"constant",regex:"^={2,}(?: +={2,})+$"},{token:"constant",regex:"^\\+-{2,}(?:\\+-{2,})+\\+$"},{token:"constant",regex:"^\\+={2,}(?:\\+={2,})+\\+$"},{token:["text",e_literal],regex:i+"(``)(?=\\S)",next:"code"},{token:["text","markup.bold"],regex:i+"(\\*\\*)(?=\\S)",next:"bold"},{token:["text","markup.italic"],regex:i+"(\\*)(?=\\S)",next:"italic"},{token:"string",regex:"\\|[\\w\\-]+?\\|"},{token:"string",regex:":[\\w-:]+:`\\S",next:"entity"},{token:["text","string"],regex:i+"(_`)(?=\\S)",next:"entity"},{token:"string",regex:"_[A-Za-z0-9\\-]+?"},{token:["text",e_link],regex:i+"(`)(?=\\S)",next:"link"},{token:e_link,regex:"[A-Za-z0-9\\-]+?__?"},{token:e_link,regex:"\\[[^\\]]+?\\]_"},{token:e_link,regex:"https?://\\S+"},{token:"constant",regex:"\\|"}],codeblock:[{token:e_literal,regex:"^ +.+$",next:"codeblock"},{token:e_literal,regex:"^$",next:"codeblock"},{token:"empty",regex:"",next:"start"}],code:[{token:e_literal,regex:"\\S``"+o,next:"start"},{defaultToken:e_literal}],bold:[{token:"markup.bold",regex:"\\S\\*\\*"+o,next:"start"},{defaultToken:"markup.bold"}],italic:[{token:"markup.italic",regex:"\\S\\*"+o,next:"start"},{defaultToken:"markup.italic"}],entity:[{token:"string",regex:"\\S`"+o,next:"start"},{defaultToken:"string"}],link:[{token:e_link,regex:"\\S`__?"+o,next:"start"},{defaultToken:e_link}],comment:[{token:"comment",regex:"^ +.+$",next:"comment"},{token:"comment",regex:"^$",next:"comment"},{token:"empty",regex:"",next:"start"}]}};r.inherits(n,x),l.RSTHighlightRules=n}),ace.define("ace/mode/rst",["require","exports","module","ace/lib/oop","ace/mode/text","ace/mode/rst_highlight_rules"],function(t,l,g){"use strict";var r=t("../lib/oop"),a=t("./text").Mode,x=t("./rst_highlight_rules").RSTHighlightRules,n=function(){this.HighlightRules=x};r.inherits(n,a),function(){this.type="text",this.$id="ace/mode/rst",this.snippetFileId="ace/snippets/rst"}.call(n.prototype),l.Mode=n}),ace.require(["ace/mode/rst"],function(t){"object"==typeof module&&"object"==typeof exports&&module&&(module.exports=t)});