ace.define("ace/ext/linking",["require","exports","module","ace/editor","ace/config"],function(v,o,s){var f=v("../editor").Editor;function c(i){var e=i.editor;if(i.getAccelKey()){e=i.editor;var n=i.getDocumentPosition(),r=e.session.getTokenAt(n.row,n.column);o.previousLinkingHover&&o.previousLinkingHover!=r&&e._emit("linkHoverOut"),e._emit("linkHover",{position:n,token:r}),o.previousLinkingHover=r}else o.previousLinkingHover&&(e._emit("linkHoverOut"),o.previousLinkingHover=!1)}function l(i){var e=i.getAccelKey();if(0==i.getButton()&&e){var n=i.editor,t=i.getDocumentPosition(),a=n.session.getTokenAt(t.row,t.column);n._emit("linkClick",{position:t,token:a})}}v("../config").defineOptions(f.prototype,"editor",{enableLinking:{set:function(i){i?(this.on("click",l),this.on("mousemove",c)):(this.off("click",l),this.off("mousemove",c))},value:!1}}),o.previousLinkingHover=!1}),ace.require(["ace/ext/linking"],function(v){"object"==typeof module&&"object"==typeof exports&&module&&(module.exports=v)});