ace.define("ace/mode/diff_highlight_rules",["require","exports","module","ace/lib/oop","ace/mode/text_highlight_rules"],function(e,n,g){"use strict";var a=e("../lib/oop"),d=e("./text_highlight_rules").TextHighlightRules,i=function(){this.$rules={start:[{regex:"^(?:\\*{15}|={67}|-{3}|\\+{3})$",token:"punctuation.definition.separator.diff",name:"keyword"},{regex:"^(@@)(\\s*.+?\\s*)(@@)(.*)$",token:["constant","constant.numeric","constant","comment.doc.tag"]},{regex:"^(\\d+)([,\\d]+)(a|d|c)(\\d+)([,\\d]+)(.*)$",token:["constant.numeric","punctuation.definition.range.diff","constant.function","constant.numeric","punctuation.definition.range.diff","invalid"],name:"meta."},{regex:"^(\\-{3}|\\+{3}|\\*{3})( .+)$",token:["constant.numeric","meta.tag"]},{regex:"^([!+>])(.*?)(\\s*)$",token:["support.constant","text","invalid"]},{regex:"^([<\\-])(.*?)(\\s*)$",token:["support.function","string","invalid"]},{regex:"^(diff)(\\s+--\\w+)?(.+?)( .+)?$",token:["variable","variable","keyword","variable"]},{regex:"^Index.+$",token:"variable"},{regex:"^\\s+$",token:"text"},{regex:"\\s*$",token:"invalid"},{defaultToken:"invisible",caseInsensitive:!0}]}};a.inherits(i,d),n.DiffHighlightRules=i}),ace.define("ace/mode/folding/diff",["require","exports","module","ace/lib/oop","ace/mode/folding/fold_mode","ace/range"],function(e,n,g){"use strict";var a=e("../../lib/oop"),d=e("./fold_mode").FoldMode,i=e("../../range").Range,f=n.FoldMode=function(t,c){this.regExpList=t,this.flag=c,this.foldingStartMarker=RegExp("^("+t.join("|")+")",this.flag)};a.inherits(f,d),function(){this.getFoldWidgetRange=function(t,c,o){for(var l=t.getLine(o),r={row:o,column:l.length},u=this.regExpList,s=1;s<=u.length;s++){var h=RegExp("^("+u.slice(0,s).join("|")+")",this.flag);if(h.test(l))break}for(var p=t.getLength();++o<p&&(l=t.getLine(o),!h.test(l)););if(o!=r.row+1)return new i(r.row,r.column,o-1,l.length)}}.call(f.prototype)}),ace.define("ace/mode/diff",["require","exports","module","ace/lib/oop","ace/mode/text","ace/mode/diff_highlight_rules","ace/mode/folding/diff"],function(e,n,g){"use strict";var a=e("../lib/oop"),d=e("./text").Mode,i=e("./diff_highlight_rules").DiffHighlightRules,f=e("./folding/diff").FoldMode,t=function(){this.HighlightRules=i,this.foldingRules=new f(["diff","@@|\\*{5}"],"i")};a.inherits(t,d),function(){this.$id="ace/mode/diff",this.snippetFileId="ace/snippets/diff"}.call(t.prototype),n.Mode=t}),ace.require(["ace/mode/diff"],function(e){"object"==typeof module&&"object"==typeof exports&&module&&(module.exports=e)});