ace.define("ace/mode/toml_highlight_rules",["require","exports","module","ace/lib/oop","ace/mode/text_highlight_rules"],function(e,n,u){"use strict";var a=e("../lib/oop"),r=e("./text_highlight_rules").TextHighlightRules,o=function(){var i=this.createKeywordMapper({"constant.language.boolean":"true|false"},"identifier");this.$rules={start:[{token:"comment.toml",regex:/#.*$/},{token:"string",regex:'"(?=.)',next:"qqstring"},{token:["variable.keygroup.toml"],regex:"(?:^\\s*)(\\[\\[([^\\]]+)\\]\\])"},{token:["variable.keygroup.toml"],regex:"(?:^\\s*)(\\[([^\\]]+)\\])"},{token:i,regex:"[a-zA-Z\\$_\xa1-\uffff][a-zA-Z\\d\\$_\xa1-\uffff]*\\b"},{token:"support.date.toml",regex:"\\d{4}-\\d{2}-\\d{2}(T)\\d{2}:\\d{2}:\\d{2}(Z)"},{token:"constant.numeric.toml",regex:"-?\\d+(\\.?\\d+)?"}],qqstring:[{token:"string",regex:"\\\\$",next:"qqstring"},{token:"constant.language.escape",regex:'\\\\[0tnr"\\\\]'},{token:"string",regex:'"|$',next:"start"},{defaultToken:"string"}]}};a.inherits(o,r),n.TomlHighlightRules=o}),ace.define("ace/mode/folding/ini",["require","exports","module","ace/lib/oop","ace/range","ace/mode/folding/fold_mode"],function(e,n,u){"use strict";var a=e("../../lib/oop"),r=e("../../range").Range,o=e("./fold_mode").FoldMode,i=n.FoldMode=function(){};a.inherits(i,o),function(){this.foldingStartMarker=/^\s*\[([^\])]*)]\s*(?:$|[;#])/,this.getFoldWidgetRange=function(t,x,l){var h=this.foldingStartMarker,d=t.getLine(l),g=d.match(h);if(g){for(var c=g[1]+".",f=d.length,p=t.getLength(),m=l,s=l;++l<p;)if(d=t.getLine(l),!/^\s*$/.test(d)){if((g=d.match(h))&&0!==g[1].lastIndexOf(c,0))break;s=l}if(s>m){var v=t.getLine(s).length;return new r(m,f,s,v)}}}}.call(i.prototype)}),ace.define("ace/mode/toml",["require","exports","module","ace/lib/oop","ace/mode/text","ace/mode/toml_highlight_rules","ace/mode/folding/ini"],function(e,n,u){"use strict";var a=e("../lib/oop"),r=e("./text").Mode,o=e("./toml_highlight_rules").TomlHighlightRules,i=e("./folding/ini").FoldMode,t=function(){this.HighlightRules=o,this.foldingRules=new i,this.$behaviour=this.$defaultBehaviour};a.inherits(t,r),function(){this.lineCommentStart="#",this.$id="ace/mode/toml"}.call(t.prototype),n.Mode=t}),ace.require(["ace/mode/toml"],function(e){"object"==typeof module&&"object"==typeof exports&&module&&(module.exports=e)});