ace.define("ace/ext/simple_tokenizer",["require","exports","module","ace/tokenizer","ace/layer/text_util"],function(r,f,a){"use strict";var u=r("../tokenizer").Tokenizer,c=r("../layer/text_util").isTextToken,l=function(){function i(e,t){this._lines=e.split(/\r\n|\r|\n/),this._states=[],this._tokenizer=t}return i.prototype.getTokens=function(e){var n=this._tokenizer.getLineTokens(this._lines[e],this._states[e-1]);return this._states[e]=n.state,n.tokens},i.prototype.getLength=function(){return this._lines.length},i}();a.exports={tokenize:function p(i,e){for(var t=new l(i,new u(e.getRules())),o=[],n=0;n<t.getLength();n++){var k=t.getTokens(n);o.push(k.map(function(s){return{className:c(s.type)?void 0:"ace_"+s.type.replace(/\./g," ace_"),value:s.value}}))}return o}}}),ace.require(["ace/ext/simple_tokenizer"],function(r){"object"==typeof module&&"object"==typeof exports&&module&&(module.exports=r)});