ace.define("ace/mode/cuttlefish_highlight_rules",["require","exports","module","ace/lib/oop","ace/mode/text_highlight_rules"],function(e,i,s){"use strict";var o=e("../lib/oop"),l=e("./text_highlight_rules").TextHighlightRules,t=function(){this.$rules={start:[{token:["text","comment"],regex:/^([ \t]*)(#.*)$/},{token:["text","keyword","text","string","text","comment"],regex:/^([ \t]*)(include)([ \t]*)([A-Za-z0-9-\_\.\*\/]+)([ \t]*)(#.*)?$/},{token:["text","keyword","text","operator","text","string","text","comment"],regex:/^([ \t]*)([A-Za-z0-9-_]+(?:\.[A-Za-z0-9-_]+)*)([ \t]*)(=)([ \t]*)([^ \t#][^#]*?)([ \t]*)(#.*)?$/},{defaultToken:"invalid"}]},this.normalizeRules()};t.metaData={fileTypes:["conf"],keyEquivalent:"^~C",name:"Cuttlefish",scopeName:"source.conf"},o.inherits(t,l),i.CuttlefishHighlightRules=t}),ace.define("ace/mode/cuttlefish",["require","exports","module","ace/lib/oop","ace/mode/text","ace/mode/cuttlefish_highlight_rules"],function(e,i,s){"use strict";var o=e("../lib/oop"),l=e("./text").Mode,t=e("./cuttlefish_highlight_rules").CuttlefishHighlightRules,h=function(){this.HighlightRules=t,this.foldingRules=null,this.$behaviour=this.$defaultBehaviour};o.inherits(h,l),function(){this.lineCommentStart="#",this.blockComment=null,this.$id="ace/mode/cuttlefish"}.call(h.prototype),i.Mode=h}),ace.require(["ace/mode/cuttlefish"],function(e){"object"==typeof module&&"object"==typeof exports&&module&&(module.exports=e)});