ace.define("ace/mode/haskell_cabal_highlight_rules",["require","exports","module","ace/lib/oop","ace/mode/text_highlight_rules"],function(a,d,f){"use strict";var g=a("../lib/oop"),h=a("./text_highlight_rules").TextHighlightRules,n=function(){this.$rules={start:[{token:"comment",regex:"^\\s*--.*$"},{token:["keyword"],regex:/^(\s*\w.*?)(:(?:\s+|$))/},{token:"constant.numeric",regex:/[\d_]+(?:(?:[\.\d_]*)?)/},{token:"constant.language.boolean",regex:"(?:true|false|TRUE|FALSE|True|False|yes|no)\\b"},{token:"markup.heading",regex:/^(\w.*)$/}]}};g.inherits(n,h),d.CabalHighlightRules=n}),ace.define("ace/mode/folding/haskell_cabal",["require","exports","module","ace/lib/oop","ace/mode/folding/fold_mode","ace/range"],function(a,d,f){"use strict";var g=a("../../lib/oop"),h=a("./fold_mode").FoldMode,n=a("../../range").Range,r=d.FoldMode=function(){};g.inherits(r,h),function(){this.isHeading=function(t,o){var i=t.getTokens(o)[0];return 0==o||i&&0===i.type.lastIndexOf("markup.heading",0)},this.getFoldWidget=function(t,o,e){if(this.isHeading(t,e))return"start";if("markbeginend"===o&&!/^\s*$/.test(t.getLine(e))){for(var i=t.getLength();++e<i&&/^\s*$/.test(t.getLine(e)););if(e==i||this.isHeading(t,e))return"end"}return""},this.getFoldWidgetRange=function(t,o,e){var c=(i=t.getLine(e)).length,m=t.getLength(),u=e,l=e;if(this.isHeading(t,e)){for(;++e<m;)if(this.isHeading(t,e)){e--;break}if((l=e)>u)for(;l>u&&/^\s*$/.test(t.getLine(l));)l--;if(l>u){var s=t.getLine(l).length;return new n(u,c,l,s)}}else if("end"===this.getFoldWidget(t,o,e)){for(s=t.getLine(l=e).length;--e>=0&&!this.isHeading(t,e););var i=t.getLine(e);return new n(e,c=i.length,l,s)}}}.call(r.prototype)}),ace.define("ace/mode/haskell_cabal",["require","exports","module","ace/lib/oop","ace/mode/text","ace/mode/haskell_cabal_highlight_rules","ace/mode/folding/haskell_cabal"],function(a,d,f){"use strict";var g=a("../lib/oop"),h=a("./text").Mode,n=a("./haskell_cabal_highlight_rules").CabalHighlightRules,r=a("./folding/haskell_cabal").FoldMode,t=function(){this.HighlightRules=n,this.foldingRules=new r,this.$behaviour=this.$defaultBehaviour};g.inherits(t,h),function(){this.lineCommentStart="--",this.blockComment=null,this.$id="ace/mode/haskell_cabal"}.call(t.prototype),d.Mode=t}),ace.require(["ace/mode/haskell_cabal"],function(a){"object"==typeof module&&"object"==typeof exports&&module&&(module.exports=a)});