ace.define("ace/ext/statusbar",["require","exports","module","ace/lib/dom","ace/lib/lang"],function(a,c,f){"use strict";var d=a("../lib/dom"),r=a("../lib/lang"),m=function(){function i(t,n){this.element=d.createElement("div"),this.element.className="ace_status-indicator",this.element.style.cssText="display: inline-block;",n.appendChild(this.element);var e=r.delayedCall(function(){this.updateStatus(t)}.bind(this)).schedule.bind(null,100);t.on("changeStatus",e),t.on("changeSelection",e),t.on("keyboardActivity",e)}return i.prototype.updateStatus=function(t){var n=[];function e(u,p){u&&n.push(u,p||"|")}e(t.keyBinding.getStatusText(t)),t.commands.recording&&e("REC");var s=t.selection,l=s.lead;if(!s.isEmpty()){var o=t.getSelectionRange();e("("+(o.end.row-o.start.row)+":"+(o.end.column-o.start.column)+")"," ")}e(l.row+":"+l.column," "),s.rangeCount&&e("["+s.rangeCount+"]"," "),n.pop(),this.element.textContent=n.join("")},i}();c.StatusBar=m}),ace.require(["ace/ext/statusbar"],function(a){"object"==typeof module&&"object"==typeof exports&&module&&(module.exports=a)});