ace.define("ace/mode/gcode_highlight_rules",["require","exports","module","ace/lib/oop","ace/mode/text_highlight_rules"],function(e,i,g){"use strict";var r=e("../lib/oop"),n=e("./text_highlight_rules").TextHighlightRules,o=function(){var l=this.createKeywordMapper({"support.function":"ATAN|ABS|ACOS|ASIN|SIN|COS|EXP|FIX|FUP|ROUND|LN|TAN",keyword:"IF|DO|WHILE|ENDWHILE|CALL|ENDIF|SUB|ENDSUB|GOTO|REPEAT|ENDREPEAT|CALL","constant.language":"PI"},"identifier",!0);this.$rules={start:[{token:"comment",regex:"\\(.*\\)"},{token:"comment",regex:"([N])([0-9]+)"},{token:"string",regex:"([G])([0-9]+\\.?[0-9]?)"},{token:"string",regex:"([M])([0-9]+\\.?[0-9]?)"},{token:"constant.numeric",regex:"([-+]?([0-9]*\\.?[0-9]+\\.?))|(\\b0[xX][a-fA-F0-9]+|(\\b\\d+(\\.\\d*)?|\\.\\d+)([eE][-+]?\\d+)?)"},{token:l,regex:"[A-Z]"},{token:"keyword.operator",regex:"EQ|LT|GT|NE|GE|LE|OR|XOR"},{token:"paren.lparen",regex:"[\\[]"},{token:"paren.rparen",regex:"[\\]]"},{token:"text",regex:"\\s+"}]}};r.inherits(o,n),i.GcodeHighlightRules=o}),ace.define("ace/mode/gcode",["require","exports","module","ace/lib/oop","ace/mode/text","ace/mode/gcode_highlight_rules","ace/range"],function(e,i,g){"use strict";var r=e("../lib/oop"),n=e("./text").Mode,o=e("./gcode_highlight_rules").GcodeHighlightRules,t=(e("../range"),function(){this.HighlightRules=o,this.$behaviour=this.$defaultBehaviour});r.inherits(t,n),function(){this.$id="ace/mode/gcode"}.call(t.prototype),i.Mode=t}),ace.require(["ace/mode/gcode"],function(e){"object"==typeof module&&"object"==typeof exports&&module&&(module.exports=e)});