ace.define("ace/theme/textmate",["require","exports","module","ace/theme/textmate-css","ace/lib/dom"],function(t,e,c){"use strict";e.isDark=!1,e.cssClass="ace-tm",e.cssText=t("./textmate-css"),e.$id="ace/theme/textmate",t("../lib/dom").importCssString(e.cssText,e.cssClass,!1)}),ace.require(["ace/theme/textmate"],function(t){"object"==typeof module&&"object"==typeof exports&&module&&(module.exports=t)});