ace.define("ace/mode/ini_highlight_rules",["require","exports","module","ace/lib/oop","ace/mode/text_highlight_rules"],function(e,a,s){"use strict";var l=e("../lib/oop"),r=e("./text_highlight_rules").TextHighlightRules,t="\\\\(?:[\\\\0abtrn;#=:]|x[a-fA-F\\d]{4})",i=function(){this.$rules={start:[{token:"punctuation.definition.comment.ini",regex:"#.*",push_:[{token:"comment.line.number-sign.ini",regex:"$|^",next:"pop"},{defaultToken:"comment.line.number-sign.ini"}]},{token:"punctuation.definition.comment.ini",regex:";.*",push_:[{token:"comment.line.semicolon.ini",regex:"$|^",next:"pop"},{defaultToken:"comment.line.semicolon.ini"}]},{token:["keyword.other.definition.ini","text","punctuation.separator.key-value.ini"],regex:"\\b([a-zA-Z0-9_.-]+)\\b(\\s*)(=)"},{token:["punctuation.definition.entity.ini","constant.section.group-title.ini","punctuation.definition.entity.ini"],regex:"^(\\[)(.*?)(\\])"},{token:"punctuation.definition.string.begin.ini",regex:"'",push:[{token:"punctuation.definition.string.end.ini",regex:"'",next:"pop"},{token:"constant.language.escape",regex:t},{defaultToken:"string.quoted.single.ini"}]},{token:"punctuation.definition.string.begin.ini",regex:'"',push:[{token:"constant.language.escape",regex:t},{token:"punctuation.definition.string.end.ini",regex:'"',next:"pop"},{defaultToken:"string.quoted.double.ini"}]}]},this.normalizeRules()};i.metaData={fileTypes:["ini","conf"],keyEquivalent:"^~I",name:"Ini",scopeName:"source.ini"},l.inherits(i,r),a.IniHighlightRules=i}),ace.define("ace/mode/folding/ini",["require","exports","module","ace/lib/oop","ace/range","ace/mode/folding/fold_mode"],function(e,a,s){"use strict";var l=e("../../lib/oop"),r=e("../../range").Range,t=e("./fold_mode").FoldMode,i=a.FoldMode=function(){};l.inherits(i,t),function(){this.foldingStartMarker=/^\s*\[([^\])]*)]\s*(?:$|[;#])/,this.getFoldWidgetRange=function(n,v,o){var c=this.foldingStartMarker,u=n.getLine(o),d=u.match(c);if(d){for(var m=d[1]+".",f=u.length,p=n.getLength(),h=o,g=o;++o<p;)if(u=n.getLine(o),!/^\s*$/.test(u)){if((d=u.match(c))&&0!==d[1].lastIndexOf(m,0))break;g=o}if(g>h){var x=n.getLine(g).length;return new r(h,f,g,x)}}}}.call(i.prototype)}),ace.define("ace/mode/ini",["require","exports","module","ace/lib/oop","ace/mode/text","ace/mode/ini_highlight_rules","ace/mode/folding/ini"],function(e,a,s){"use strict";var l=e("../lib/oop"),r=e("./text").Mode,t=e("./ini_highlight_rules").IniHighlightRules,i=e("./folding/ini").FoldMode,n=function(){this.HighlightRules=t,this.foldingRules=new i,this.$behaviour=this.$defaultBehaviour};l.inherits(n,r),function(){this.lineCommentStart=";",this.blockComment=null,this.$id="ace/mode/ini"}.call(n.prototype),a.Mode=n}),ace.require(["ace/mode/ini"],function(e){"object"==typeof module&&"object"==typeof exports&&module&&(module.exports=e)});