ace.define("ace/mode/plain_text",["require","exports","module","ace/lib/oop","ace/mode/text","ace/mode/text_highlight_rules","ace/mode/behaviour"],function(e,o,n){"use strict";var i=e("../lib/oop"),a=e("./text").Mode,h=e("./text_highlight_rules").TextHighlightRules,l=e("./behaviour").Behaviour,t=function(){this.HighlightRules=h,this.$behaviour=new l};i.inherits(t,a),function(){this.type="text",this.getNextLineIndent=function(u,c,d){return""},this.$id="ace/mode/plain_text"}.call(t.prototype),o.Mode=t}),ace.require(["ace/mode/plain_text"],function(e){"object"==typeof module&&"object"==typeof exports&&module&&(module.exports=e)});