ace.define("ace/ext/static-css",["require","exports","module"],function(l,S,v){v.exports=".ace_static_highlight {\n    font-family: 'Monaco', 'Menlo', 'Ubuntu Mono', 'Consolas', 'Source Code Pro', 'source-code-pro', 'Droid Sans Mono', monospace;\n    font-size: 12px;\n    white-space: pre-wrap\n}\n\n.ace_static_highlight .ace_gutter {\n    width: 2em;\n    text-align: right;\n    padding: 0 3px 0 0;\n    margin-right: 3px;\n    contain: none;\n}\n\n.ace_static_highlight.ace_show_gutter .ace_line {\n    padding-left: 2.6em;\n}\n\n.ace_static_highlight .ace_line { position: relative; }\n\n.ace_static_highlight .ace_gutter-cell {\n    -moz-user-select: -moz-none;\n    -khtml-user-select: none;\n    -webkit-user-select: none;\n    user-select: none;\n    top: 0;\n    bottom: 0;\n    left: 0;\n    position: absolute;\n}\n\n\n.ace_static_highlight .ace_gutter-cell:before {\n    content: counter(ace_line, decimal);\n    counter-increment: ace_line;\n}\n.ace_static_highlight {\n    counter-reset: ace_line;\n}\n"}),ace.define("ace/ext/static_highlight",["require","exports","module","ace/edit_session","ace/layer/text","ace/ext/static-css","ace/config","ace/lib/dom","ace/lib/lang"],function(l,S,v){"use strict";var x=l("../edit_session").EditSession,N=l("../layer/text").Text,M=l("./static-css"),y=l("../config"),T=l("../lib/dom"),L=l("../lib/lang").escapeHTML,C=function(){function t(e){this.type=e,this.style={},this.textContent=""}return t.prototype.cloneNode=function(){return this},t.prototype.appendChild=function(e){this.textContent+=e.toString()},t.prototype.toString=function(){var e=[];if("fragment"!=this.type){e.push("<",this.type),this.className&&e.push(" class='",this.className,"'");var a=[];for(var s in this.style)a.push(s,":",this.style[s]);a.length&&e.push(" style='",a.join(""),"'"),e.push(">")}return this.textContent&&e.push(this.textContent),"fragment"!=this.type&&e.push("</",this.type,">"),e.join("")},t}(),g={createTextNode:function(t,e){return L(t)},createElement:function(t){return new C(t)},createFragment:function(){return new C("fragment")}},b=function(){this.config={},this.dom=g};b.prototype=N.prototype;var d=function(t,e,a){var s=t.className.match(/lang-(\w+)/),u=e.mode||s&&"ace/mode/"+s[1];if(!u)return!1;var o=e.theme||"ace/theme/textmate",n="",h=[];if(t.firstElementChild)for(var c=0,r=0;r<t.childNodes.length;r++){var i=t.childNodes[r];3==i.nodeType?(c+=i.data.length,n+=i.data):h.push(c,i)}else n=t.textContent,e.trim&&(n=n.trim());d.render(n,u,o,e.firstLineNumber,!e.showGutter,function(f){T.importCssString(f.css,"ace_highlight"),t.innerHTML=f.html;for(var m=t.firstChild.firstChild,p=0;p<h.length;p+=2){var _=f.session.doc.indexToPosition(h[p]),w=m.children[_.row];w&&w.appendChild(h[p+1])}a&&a()})};d.render=function(t,e,a,s,u,o){var c,n=1,h=x.prototype.$modes;function r(){var i=d.renderSync(t,e,a,s,u);return o?o(i):i}return"string"==typeof a&&(n++,y.loadModule(["theme",a],function(i){a=i,--n||r()})),e&&"object"==typeof e&&!e.getTokenizer&&(e=(c=e).path),"string"==typeof e&&(n++,y.loadModule(["mode",e],function(i){(!h[e]||c)&&(h[e]=new i.Mode(c)),e=h[e],--n||r()})),--n||r()},d.renderSync=function(t,e,a,s,u){s=parseInt(s||1,10);var o=new x("");o.setUseWorker(!1),o.setMode(e);var n=new b;n.setSession(o),Object.keys(n.$tabStrings).forEach(function(p){if("string"==typeof n.$tabStrings[p]){var _=g.createFragment();_.textContent=n.$tabStrings[p],n.$tabStrings[p]=_}}),o.setValue(t);var h=o.getLength(),c=g.createElement("div");c.className=a.cssClass;var r=g.createElement("div");r.className="ace_static_highlight"+(u?"":" ace_show_gutter"),r.style["counter-reset"]="ace_line "+(s-1);for(var i=0;i<h;i++){var f=g.createElement("div");if(f.className="ace_line",!u){var m=g.createElement("span");m.className="ace_gutter ace_gutter-cell",m.textContent="",f.appendChild(m)}n.$renderLine(f,i,!1),f.textContent+="\n",r.appendChild(f)}return c.appendChild(r),{css:M+a.cssText,html:c.toString(),session:o}},v.exports=d,v.exports.highlight=d}),ace.require(["ace/ext/static_highlight"],function(l){"object"==typeof module&&"object"==typeof exports&&module&&(module.exports=l)});