ace.define("ace/mode/folding/coffee",["require","exports","module","ace/lib/oop","ace/mode/folding/fold_mode","ace/range"],function(i,d,v){"use strict";var g=i("../../lib/oop"),c=i("./fold_mode").FoldMode,r=i("../../range").Range,u=d.FoldMode=function(){};g.inherits(u,c),function(){this.getFoldWidgetRange=function(e,m,t){var n=this.indentationBlock(e,t);if(n)return n;var a=/\S/,o=e.getLine(t),f=o.search(a);if(-1!=f&&"#"==o[f]){for(var l=o.length,h=e.getLength(),s=t,p=t;++t<h;){var x=(o=e.getLine(t)).search(a);if(-1!=x){if("#"!=o[x])break;p=t}}if(p>s){var _=e.getLine(p).length;return new r(s,l,p,_)}}},this.getFoldWidget=function(e,m,t){var n=e.getLine(t),a=n.search(/\S/),o=e.getLine(t+1),f=e.getLine(t-1),l=f.search(/\S/),h=o.search(/\S/);if(-1==a)return e.foldWidgets[t-1]=-1!=l&&l<h?"start":"","";if(-1==l){if(a==h&&"#"==n[a]&&"#"==o[a])return e.foldWidgets[t-1]="",e.foldWidgets[t+1]="","start"}else if(l==a&&"#"==n[a]&&"#"==f[a]&&-1==e.getLine(t-2).search(/\S/))return e.foldWidgets[t-1]="start",e.foldWidgets[t+1]="","";return e.foldWidgets[t-1]=-1!=l&&l<a?"start":"",a<h?"start":""}}.call(u.prototype)}),ace.define("ace/mode/space_highlight_rules",["require","exports","module","ace/lib/oop","ace/mode/text_highlight_rules"],function(i,d,v){"use strict";var g=i("../lib/oop"),c=i("./text_highlight_rules").TextHighlightRules,r=function(){this.$rules={start:[{token:"empty_line",regex:/ */,next:"key"},{token:"empty_line",regex:/$/,next:"key"}],key:[{token:"variable",regex:/\S+/},{token:"empty_line",regex:/$/,next:"start"},{token:"keyword.operator",regex:/ /,next:"value"}],value:[{token:"keyword.operator",regex:/$/,next:"start"},{token:"string",regex:/[^$]/}]}};g.inherits(r,c),d.SpaceHighlightRules=r}),ace.define("ace/mode/space",["require","exports","module","ace/lib/oop","ace/mode/text","ace/mode/folding/coffee","ace/mode/space_highlight_rules"],function(i,d,v){"use strict";var g=i("../lib/oop"),c=i("./text").Mode,r=i("./folding/coffee").FoldMode,u=i("./space_highlight_rules").SpaceHighlightRules,e=function(){this.HighlightRules=u,this.foldingRules=new r,this.$behaviour=this.$defaultBehaviour};g.inherits(e,c),function(){this.$id="ace/mode/space"}.call(e.prototype),d.Mode=e}),ace.require(["ace/mode/space"],function(i){"object"==typeof module&&"object"==typeof exports&&module&&(module.exports=i)});